@@ -19,20 +19,31 @@ namespace i2p
 {
 namespace transport
 {
+	const int UPNP_RESPONSE_TIMEOUT = 2000; // in milliseconds
+
+	enum
+	{
+		UPNP_IGD_NONE = 0,
+		UPNP_IGD_VALID_CONNECTED = 1,
+		UPNP_IGD_VALID_NOT_CONNECTED = 2,
+		UPNP_IGD_INVALID = 3
+	};
+
 	class UPnP
 	{
-	public:
+		public:
 
 		UPnP ();
 		~UPnP ();
-        void Close ();
+		void Close ();
 
-        void Start ();
-        void Stop ();
+		void Start ();
+		void Stop ();
 
-	private:
+		private:
 
 		void Discover ();
+		int  CheckMapping (const char* port, const char* type);
 		void PortMapping ();
 		void TryPortMapping (std::shared_ptr<i2p::data::RouterInfo::Address> address);
 		void CloseMapping ();
@@ -41,23 +52,22 @@ namespace transport
 		void Run ();
 		std::string GetProto (std::shared_ptr<i2p::data::RouterInfo::Address> address);
 
-	private:
+		private:
 
 		bool m_IsRunning;
-        std::unique_ptr<std::thread> m_Thread;
+		std::unique_ptr<std::thread> m_Thread;
 		std::condition_variable m_Started;
 		std::mutex m_StartedMutex;
 		boost::asio::io_service m_Service;
 		boost::asio::deadline_timer m_Timer;
-        struct UPNPUrls m_upnpUrls;
-        struct IGDdatas m_upnpData;
+        bool m_upnpUrlsInitialized=false;
+		struct UPNPUrls m_upnpUrls;
+		struct IGDdatas m_upnpData;
 
-        // For miniupnpc
-        char * m_MulticastIf = 0;
-        char * m_Minissdpdpath = 0;
-        struct UPNPDev * m_Devlist = 0;
-        char m_NetworkAddr[64];
-        char m_externalIPAddress[40];
+		// For miniupnpc
+		struct UPNPDev * m_Devlist = 0;
+		char m_NetworkAddr[64];
+		char m_externalIPAddress[40];
 	};
 }
 }
@@ -65,14 +75,15 @@ namespace transport
 #else  // USE_UPNP
 namespace i2p {
 namespace transport {
-  /* class stub */
-  class UPnP {
-  public:
-    UPnP () {};
-    ~UPnP () {};
-    void Start () { LogPrint(eLogWarning, "UPnP: this module was disabled at compile-time"); }
-    void Stop () {};
-  };
+	/* class stub */
+	class UPnP {
+		public:
+
+		UPnP () {};
+		~UPnP () {};
+		void Start () { LogPrint(eLogWarning, "UPnP: this module was disabled at compile-time"); }
+		void Stop () {};
+	};
 }
 }
 #endif // USE_UPNP
