@@ -43,8 +43,8 @@ namespace data
 	const char * GetBase64SubstitutionTable ()
 	{
 		return T64;
-	}	
-	
+	}
+
 	/*
 	* Reverse Substitution Table (built in run time)
 	*/
@@ -53,10 +53,10 @@ namespace data
 	static int isFirstTime = 1;
 
 	/*
-	* Padding 
+	* Padding
 	*/
 
-	static char P64 = '='; 
+	static char P64 = '=';
 
 	/*
 	*
@@ -68,11 +68,11 @@ namespace data
 	*/
 
 	size_t                                /* Number of bytes in the encoded buffer */
-	ByteStreamToBase64 ( 
+	ByteStreamToBase64 (
 		    const uint8_t * InBuffer,           /* Input buffer, binary data */
-		    size_t    InCount,              /* Number of bytes in the input buffer */ 
+		    size_t    InCount,              /* Number of bytes in the input buffer */
 		    char  * OutBuffer,          /* output buffer */
-		size_t len			   /* length of output buffer */	             
+		size_t len			   /* length of output buffer */
 	)
 
 	{
@@ -80,9 +80,9 @@ namespace data
 		unsigned char * pd;
 		unsigned char   acc_1;
 		unsigned char   acc_2;
-		int             i; 
-		int             n; 
-		int             m; 
+		int             i;
+		int             n;
+		int             m;
 		size_t outCount;
 
 		ps = (unsigned char *)InBuffer;
@@ -96,7 +96,7 @@ namespace data
 		pd = (unsigned char *)OutBuffer;
 		for ( i = 0; i<n; i++ ){
 		     acc_1 = *ps++;
-		     acc_2 = (acc_1<<4)&0x30; 
+		     acc_2 = (acc_1<<4)&0x30;
 		     acc_1 >>= 2;              /* base64 digit #1 */
 		     *pd++ = T64[acc_1];
 		     acc_1 = *ps++;
@@ -109,7 +109,7 @@ namespace data
 		     *pd++ = T64[acc_1];
 		     acc_2 &= 0x3f;            /* base64 digit #4 */
 		     *pd++ = T64[acc_2];
-		} 
+		}
 		if ( m == 1 ){
 		     acc_1 = *ps++;
 		     acc_2 = (acc_1<<4)&0x3f;  /* base64 digit #2 */
@@ -122,7 +122,7 @@ namespace data
 		}
 		else if ( m == 2 ){
 		     acc_1 = *ps++;
-		     acc_2 = (acc_1<<4)&0x3f; 
+		     acc_2 = (acc_1<<4)&0x3f;
 		     acc_1 >>= 2;              /* base64 digit #1 */
 		     *pd++ = T64[acc_1];
 		     acc_1 = *ps++;
@@ -133,7 +133,7 @@ namespace data
 		     *pd++ = T64[acc_1];
 		     *pd++ = P64;
 		}
-		
+
 		return outCount;
 	}
 
@@ -148,10 +148,10 @@ namespace data
 	*/
 
 	size_t                              /* Number of output bytes */
-	Base64ToByteStream ( 
+	Base64ToByteStream (
 		      const char * InBuffer,           /* BASE64 encoded buffer */
 		      size_t    InCount,          /* Number of input bytes */
-		      uint8_t  * OutBuffer,	/* output buffer length */ 	
+		      uint8_t  * OutBuffer,	/* output buffer length */
 		  size_t len         	/* length of output buffer */
 	)
 	{
@@ -159,28 +159,28 @@ namespace data
 		unsigned char * pd;
 		unsigned char   acc_1;
 		unsigned char   acc_2;
-		int             i; 
-		int             n; 
-		int             m; 
+		int             i;
+		int             n;
+		int             m;
 		size_t outCount;
 
 		if (isFirstTime) iT64Build();
 		n = InCount/4;
 		m = InCount%4;
-		if (InCount && !m) 
+		if (InCount && !m)
 		     outCount = 3*n;
 		else {
 		     outCount = 0;
 		     return 0;
 		}
-		
+
 		ps = (unsigned char *)(InBuffer + InCount - 1);
 		while ( *ps-- == P64 ) outCount--;
 		ps = (unsigned char *)InBuffer;
-		
+
 		if (outCount > len) return -1;
 		pd = OutBuffer;
-		auto endOfOutBuffer = OutBuffer + outCount;		
+		auto endOfOutBuffer = OutBuffer + outCount;
 		for ( i = 0; i < n; i++ ){
 		     acc_1 = iT64[*ps++];
 		     acc_2 = iT64[*ps++];
@@ -193,7 +193,7 @@ namespace data
 		     acc_1 = iT64[*ps++];
 		     acc_2 |= acc_1 >> 2;
 		     *pd++ = acc_2;
-			  if (pd >= endOfOutBuffer) break;	
+			  if (pd >= endOfOutBuffer) break;
 
 		     acc_2 = iT64[*ps++];
 		     acc_2 |= acc_1 << 6;
@@ -203,13 +203,28 @@ namespace data
 		return outCount;
 	}
 
-	size_t Base64EncodingBufferSize (const size_t input_size) 
+	size_t Base64EncodingBufferSize (const size_t input_size)
 	{
 		auto d = div (input_size, 3);
 		if (d.rem) d.quot++;
 		return 4*d.quot;
 	}
-	
+
+	std::string ToBase64Standard (const std::string& in)
+	{
+		auto len = Base64EncodingBufferSize (in.length ());	
+		char * str = new char[len+1];
+		auto l = ByteStreamToBase64 ((const uint8_t *)in.c_str (), in.length (), str, len);
+		str[l] = 0;
+		// replace '-' by '+' and '~' by '/'
+		for (size_t i = 0; i < l; i++)
+			if (str[i] == '-') str[i] = '+';
+			else if (str[i] == '~') str[i] = '/';
+		std::string s(str);
+		delete[] str;
+		return s;
+	}
+
 	/*
 	*
 	* iT64
@@ -228,20 +243,20 @@ namespace data
 		iT64[(int)P64] = 0;
 	}
 
-	size_t Base32ToByteStream (const char * inBuf, size_t len, uint8_t * outBuf, size_t outLen) 
+	size_t Base32ToByteStream (const char * inBuf, size_t len, uint8_t * outBuf, size_t outLen)
 	{
 		int tmp = 0, bits = 0;
 		size_t ret = 0;
 		for (size_t i = 0; i < len; i++)
 		{
-			char ch = inBuf[i];	
+			char ch = inBuf[i];
 			if (ch >= '2' && ch <= '7') // digit
 				ch = (ch - '2') + 26; // 26 means a-z
 			else if (ch >= 'a' && ch <= 'z')
 				ch = ch - 'a'; // a = 0
 			else
 				return 0; // unexpected character
-			
+
 			tmp |= ch;
 			bits += 5;
 			if (bits >= 8)
@@ -261,23 +276,23 @@ namespace data
 		size_t ret = 0, pos = 1;
 		int bits = 8, tmp = inBuf[0];
 		while (ret < outLen && (bits > 0 || pos < len))
-		{ 	
+		{
 			if (bits < 5)
 			{
 				if (pos < len)
 				{
 					tmp <<= 8;
-		      		tmp |= inBuf[pos] & 0xFF;
+					tmp |= inBuf[pos] & 0xFF;
 					pos++;
-		      		bits += 8;
+					bits += 8;
 				}
 				else // last byte
 				{
 					tmp <<= (5 - bits);
-				  	bits = 5;
+					bits = 5;
 				}
-			}	
-		
+			}
+
 			bits -= 5;
 			int ind = (tmp >> bits) & 0x1F;
 			outBuf[ret] = (ind < 26) ? (ind + 'a') : ((ind - 26) + '2');
