@@ -38,10 +38,19 @@ namespace data
 		m_IsUpdated (true), m_IsUnreachable (false), m_SupportedTransports (0), m_Caps (0)
 	{
 		m_Addresses = boost::make_shared<Addresses>(); // create empty list
-		m_Buffer = new uint8_t[MAX_RI_BUFFER_SIZE];
-		memcpy (m_Buffer, buf, len);
-		m_BufferLen = len;
-		ReadFromBuffer (true);
+		if (len <= MAX_RI_BUFFER_SIZE)
+		{	
+			m_Buffer = new uint8_t[MAX_RI_BUFFER_SIZE];
+			memcpy (m_Buffer, buf, len);
+			m_BufferLen = len;
+			ReadFromBuffer (true);
+		}
+		else
+		{
+			LogPrint (eLogError, "RouterInfo: Buffer is too long ", len, ". Ignored");
+			m_Buffer = nullptr;
+			m_IsUnreachable = true;
+		}
 	}
 
 	RouterInfo::~RouterInfo ()
@@ -49,9 +58,15 @@ namespace data
 		delete[] m_Buffer;
 	}
 
-	void RouterInfo::Update (const uint8_t * buf, int len)
+	void RouterInfo::Update (const uint8_t * buf, size_t len)
 	{
-		// verify signature since we have indentity already
+		if (len > MAX_RI_BUFFER_SIZE)
+		{
+			LogPrint (eLogError, "RouterInfo: Buffer is too long ", len);
+			m_IsUnreachable = true;
+			return;
+		}
+		// verify signature since we have identity already
 		int l = len - m_RouterIdentity->GetSignatureLen ();
 		if (m_RouterIdentity->Verify (buf, l, buf + l))
 		{
@@ -100,8 +115,7 @@ namespace data
 				return false;
 			}
 			s.seekg(0, std::ios::beg);
-			if (!m_Buffer)
-				m_Buffer = new uint8_t[MAX_RI_BUFFER_SIZE];
+			if (!m_Buffer) m_Buffer = new uint8_t[MAX_RI_BUFFER_SIZE];
 			s.read((char *)m_Buffer, m_BufferLen);
 		}
 		else
@@ -208,26 +222,21 @@ namespace data
 				{
 					boost::system::error_code ecode;
 					address->host = boost::asio::ip::address::from_string (value, ecode);
-					if (ecode)
+					if (!ecode)
 					{
-						if (address->transportStyle == eTransportNTCP)
-						{
-							supportedTransports |= eNTCPV4; // TODO:
-							address->addressString = value;
-						}
-						else
-						{
-							supportedTransports |= eSSUV4; // TODO:
-							address->addressString = value;
-						}
-					}
-					else
-					{
-						// add supported protocol
-						if (address->host.is_v4 ())
-							supportedTransports |= (address->transportStyle == eTransportNTCP) ? eNTCPV4 : eSSUV4;
-						else
-							supportedTransports |= (address->transportStyle == eTransportNTCP) ? eNTCPV6 : eSSUV6;
+#if BOOST_VERSION >= 104900
+						if (!address->host.is_unspecified ()) // check if address is valid
+#else
+						address->host.to_string (ecode);
+						if (!ecode)
+#endif
+						{	
+							// add supported protocol
+							if (address->host.is_v4 ())
+								supportedTransports |= (address->transportStyle == eTransportNTCP) ? eNTCPV4 : eSSUV4;
+							else
+								supportedTransports |= (address->transportStyle == eTransportNTCP) ? eNTCPV6 : eSSUV6;
+						}	
 					}
 				}
 				else if (!strcmp (key, "port"))
@@ -264,6 +273,11 @@ namespace data
 				else if (key[0] == 'i')
 				{
 					// introducers
+					if (!address->ssu)
+					{
+						LogPrint (eLogError, "RouterInfo: Introducer is presented for non-SSU address. Skipped");
+						continue;
+					}	
 					introducers = true;
 					size_t l = strlen(key);
 					unsigned char index = key[l-1] - '0'; // TODO:
@@ -607,15 +621,21 @@ namespace data
 		std::stringstream s;
 		uint8_t ident[1024];
 		auto identLen = privateKeys.GetPublic ()->ToBuffer (ident, 1024);
+		auto signatureLen = privateKeys.GetPublic ()->GetSignatureLen ();
 		s.write ((char *)ident, identLen);
 		WriteToStream (s);
 		m_BufferLen = s.str ().size ();
 		if (!m_Buffer)
 			m_Buffer = new uint8_t[MAX_RI_BUFFER_SIZE];
-		memcpy (m_Buffer, s.str ().c_str (), m_BufferLen);
-		// signature
-		privateKeys.Sign ((uint8_t *)m_Buffer, m_BufferLen, (uint8_t *)m_Buffer + m_BufferLen);
-		m_BufferLen += privateKeys.GetPublic ()->GetSignatureLen ();
+		if (m_BufferLen + signatureLen < MAX_RI_BUFFER_SIZE)
+		{
+			memcpy (m_Buffer, s.str ().c_str (), m_BufferLen);
+			// signature
+			privateKeys.Sign ((uint8_t *)m_Buffer, m_BufferLen, (uint8_t *)m_Buffer + m_BufferLen);
+			m_BufferLen += signatureLen;
+		}
+		else
+			LogPrint (eLogError, "RouterInfo: Our RouterInfo is too long ", m_BufferLen + signatureLen);
 	}
 
 	bool RouterInfo::SaveToFile (const std::string& fullPath)
@@ -789,6 +809,11 @@ namespace data
 			return m_SupportedTransports & (eSSUV4 | eSSUV6);
 	}
 
+	bool RouterInfo::IsSSUV6 () const
+	{
+		return m_SupportedTransports & eSSUV6;
+	}
+
 	bool RouterInfo::IsNTCP2 (bool v4only) const
 	{
 		if (v4only)
@@ -799,7 +824,7 @@ namespace data
 
 	bool RouterInfo::IsV6 () const
 	{
-		return m_SupportedTransports & (eNTCPV6 | eSSUV6);
+		return m_SupportedTransports & (eNTCPV6 | eSSUV6 | eNTCP2V6);
 	}
 
 	bool RouterInfo::IsV4 () const
@@ -888,7 +913,7 @@ namespace data
 	template<typename Filter>	
 	std::shared_ptr<const RouterInfo::Address> RouterInfo::GetAddress (Filter filter) const
 	{
-		// TODO: make it more gereric using comparator
+		// TODO: make it more generic using comparator
 #if (BOOST_VERSION >= 105300)
 		auto addresses = boost::atomic_load (&m_Addresses);
 #else
